#!/bin/sh

NOTES="/mnt/onboard/.adds/notes"
# Note: Depending on your language this folder may have a different name
EXPORT="/mnt/onboard/Exported Annotations/Notes.md"

DB="/mnt/onboard/.kobo/KoboReader.sqlite"
SQLITE="${NOTES}/sqlite3"

KC_HOME="/usr/local/kobocloud"
CURL="$KC_HOME/curl --cacert $KC_HOME/ca-bundle.crt --fail --silent"
HEADER="X-Requested-With: XMLHttpRequest"
URL="[Your Nextcloud/Owncloud instance]/public.php/webdav"
USER="[Your Share]:"

LD_LIBRARY_PATH="${NOTES}/lib:${LD_LIBRARY_PATH}"
export LD_LIBRARY_PATH

mkdir -p "$(dirname "$EXPORT")"

echo -e "# Kobo Notes\n" > $EXPORT
echo -e "*$(date -R)*\n" >> $EXPORT
echo -e "## Highlights\n" >> $EXPORT

# UTF-8 char table (decimal):
#      9: Tab
#     10: Line feed
#     32: Space
#     58: :
#     62: >
#     42: *
#     92: \
#   8230: …
#   9999: ✏
# 128278: 🔖
# 128196: 📄

SQL="SELECT TRIM(
  '### ' ||
  CASE
    WHEN b.Type = 'dogear' THEN
      char(128278, 32)
    WHEN b.Type = 'note' THEN
      char(9999, 32)
    WHEN b.Type = 'highlight' THEN
      char(128196, 32)
  END
  || c.BookTitle || ', ' || COALESCE(c1.Attribution, 'N/A') || char(10, 10, 42)
  || datetime(b.dateCreated) || char(42, 92, 10) /* force Markdown newline */
  || c.Title || char(10, 10, 62, 32) ||
  CASE
    WHEN b.Type = 'dogear' THEN
      COALESCE(ContextString, 'No context available') || char(8230, 10) /* only kepubs have context */
    ELSE
      REPLACE(              /* start Markdown quote */
        REPLACE(
          TRIM(             /* trim newlines */
            TRIM(           /* trim tabs */
              TRIM(b.Text), /* trim spaces */
              char(9)
            ),
            char(10)
          ),
          char(9), ''
        ),
        char(10), char(10, 62, 32, 10, 62, 32)) /* continue Markdown quote for multiple paragraphs */
      || char(10, 10)
      || COALESCE(b.Annotation, '') || char(10)
  END, char(10)
  ) || char(10, 10)
  FROM Bookmark b
    JOIN Content c ON b.VolumeID = c.BookID
    JOIN Content c1 ON c.BookID = c1.ContentID
  WHERE b.Hidden = 'false'
    AND ((c.MimeType NOT IN ('application/xhtml+xml', /* epub */
                             'application/x-kobo-epub+zip')
          AND c.ContentID = b.ContentID)
        OR (c.MimeType IN ('application/xhtml+xml',   /* kepub */
                           'application/x-kobo-epub+zip')
            AND c.ContentType = 899
            AND c.ContentID LIKE b.ContentID || '-%'))
  ORDER BY c.BookTitle ASC,
           c.VolumeIndex ASC,
           b.DateCreated ASC;"

$SQLITE "$DB" "$SQL" >> $EXPORT

echo -e "## Book progress\n" >> $EXPORT

echo -e "Currently reading:\n" >> $EXPORT

SQL="SELECT
  '- ' || c.Title || ', ' || COALESCE(c.Attribution, 'N/A')
  || ' (' || c.___PercentRead || '% read' || COALESCE(', ' || c1.Title, '') || ')'
  FROM (SELECT *, /* get chapter id without path */
               REPLACE(tmp, RTRIM(tmp, REPLACE(tmp, '/', '')), '') chapterIDcleaned
    FROM (SELECT *, /* get chapter id without anchor */
                 REPLACE(ChapterIDBookmarked, LTRIM(ChapterIDBookmarked, REPLACE(ChapterIDBookmarked, '#', '')), '') tmp
      FROM content)) c
    JOIN Content c1 ON c.ContentID = c1.BookID
  WHERE c.ContentType = 6
    AND c.ReadStatus = 1
    AND c.IsDownloaded = 'true'
    AND c1.ContentID LIKE '%' || chapterIDcleaned || '%'
    AND NOT (c.MimeType IN ('application/xhtml+xml',
                            'application/x-kobo-epub+zip')
             AND c1.ContentType = 9) /* kepub has chapters as both 9 and 899 */
  ORDER BY c.___PercentRead DESC,
           c.Title ASC,
           c.Attribution ASC;"

$SQLITE "$DB" "$SQL" >> $EXPORT

DATE=$(date "+%Y-%m-%d-%H-%M-%S")
$CURL -u "$USER" -H "$HEADER" -T "$EXPORT" "$URL/notes-$DATE.md"
